#!/usr/bin/python
__author__ = "Oron Peled <oron@actcom.co.il>"
__copyright__ = "2010-2016 (C) Xorcom Inc."
__revision__ = "$Id:$"
__version__ = "2.0"

import time
import os
import sys
import dbus
import dbus.service
import gobject
import dbus.glib
import xusb
import psptalk
import pyudev
# The GUDevMonitorObserver is depricated in pyudev starting from v.0.17
try:
	from pyudev.glib import GUDevMonitorObserver as MonitorObserver
except:
	from pyudev.glib import MonitorObserver

"""
Manager handles hotplug of multiple RPS devices and for each one
instanciates a Device object.
"""
class Manager(dbus.service.Object):

	__single = None

	def __init__(self, wanted_vendor='e4e4', wanted_product='1183'):

		self.psp_ifaces = {}
		# Make sure we are singleton
		if Manager.__single:
			raise "Manager already instanciated"
		Manager.__single = self 
		#xusb.set_verbose(10)
		#xusb.set_debug_mask(0xff)
		#psptalk.set_verbose(10)
		#psptalk.set_debug_mask(0xff)
		self.bus = dbus.SystemBus()
		self.bus_name = dbus.service.BusName('com.xorcom.xrps', bus=dbus.SystemBus())
		oname = "/com/xorcom/xrps/Manager"
		dbus.service.Object.__init__(self, bus_name=self.bus_name, object_path=oname)
		#print "register %s" % (oname)

		self.context = pyudev.Context()
		# Register for device notifications
		self.monitor = pyudev.Monitor.from_netlink(self.context)
		self.monitor.filter_by(subsystem='usb', device_type='usb_device')
		self.observer = MonitorObserver(self.monitor)
		#XorcomUsbDevices.observer_map[self.observer] = self
		self.observer.connect('device-event', self.device_event)
		# Enumerate devices
		for device in self.context.list_devices(subsystem='usb', ID_MODEL_ID=wanted_product):
			try:
				idVendor = device.attributes['idVendor']
				idProduct = device.attributes['idProduct']
			except:
				continue
			if wanted_vendor and wanted_vendor != idVendor:
				continue
			if wanted_product and wanted_product != idProduct:
				continue
			self.device_add(device)
		self.start()

	#@staticmethod
	def device_event(self, observer, action, device):
		#obj = XorcomUsbDevices.observer_map[observer]
		if action == 'add':
			self.device_add(device)
		elif action == 'remove':
			self.device_remove(device)
		else:
			print "Uknown action '%s'" % action

	def start(self):
			self.monitor.start()

	def device_add(self, device):
		dev_info = dict(device)
		busnum = dev_info['BUSNUM']
		devnum = dev_info['DEVNUM']
		name = busnum + '/' + devnum
		name = name.encode('ascii')
		print "Opening: " , name
		dev = xusb.find_bypath(name)
		if not dev:
			raise "xrps.Manager.device_add: No dev found for ", name
		spec = dev.spec
		print "Added device (%s)" % (name)
		ps = Device(dev)
		self.psp_ifaces[name] = ps
		self.device_added(name)

	def device_remove(self, device):
		dev_info = dict(device)
		busnum = dev_info['BUSNUM']
		devnum = dev_info['DEVNUM']
		name = busnum + '/' + devnum
		print "Removed device (%s)" % (name)
		del self.psp_ifaces[name]
		self.device_removed(name)
		print "Removed device (%s)" % (name)

	@dbus.service.method(dbus_interface='com.xorcom.xrps.Manager', out_signature='as')
	def names(self):
		namelist = []
		for n in self.psp_ifaces.keys():
			namelist.append(n)
			#print n
		return namelist

	@dbus.service.signal(dbus_interface='com.xorcom.xrps.Manager', signature='s')
	def device_added(self, name):
		pass
		#print "Manager Added: ", name

	@dbus.service.signal(dbus_interface='com.xorcom.xrps.Manager', signature='s')
	def device_removed(self, name):
		pass
		#print "Manager Removed: ", name

"""
One Device object per multi-ps device
"""
class Device(dbus.service.Object):
	def __init__(self, xusbdev):
		self.xusbdev = xusbdev
		self.devpath = xusbdev.devpath()
		self.last_voltage = [ 0, 0, 0, 0 ]
		self.iface = xusbdev.claim(1)
		self.psdev = psptalk.PSDev(self.iface)
		oname = "/com/xorcom/xrps/" + self.name()
		#print "register %s" % (oname)
		bus_name = dbus.service.BusName('com.xorcom.xrps', bus=dbus.SystemBus())
		dbus.service.Object.__init__(self, bus_name=bus_name, object_path=oname)
		self.poll_voltage()
		self.poll_timer = gobject.timeout_add(10000, self.poll_voltage)

	def poll_voltage(self):
		ret = self.psdev.refresh_volts()
		if ret < 0:
			#print "DEBUG: ret=%d\n" % (ret)
			gobject.source_remove(self.poll_timer)
			return True
		for i in range(0, 4):
			v = self.psdev.voltage(i)
			delta = v - self.last_voltage[i]
			if abs(delta) > 1:
				self.VoltageChanged(i, v)
			self.last_voltage[i] = v
		#print "Poll: ", self.name(), self.last_voltage
		return True

	@dbus.service.signal(dbus_interface='com.xorcom.xrps', signature='ii')
	def VoltageChanged(self, psnum, decivolts):
		pass
		#print "VoltageChange:%s[%i] %f\n" % (self.name(), psnum, decivolts/10),

	@dbus.service.method(dbus_interface='com.xorcom.xrps', out_signature='s')
	def name(self):
		return self.devpath

	@dbus.service.method(dbus_interface='com.xorcom.xrps', in_signature='i')
	def leds(self, i):
		return self.psdev.leds(i)

	@dbus.service.method(dbus_interface='com.xorcom.xrps', out_signature='ai')
	def voltage(self):
		#print "Voltage query", self.last_voltage
		return self.last_voltage

	@dbus.service.method(dbus_interface='com.xorcom.xrps', out_signature='i')
	def get_ps_mask(self):
		self.psdev.get_mask()
		return self.psdev.ps_mask()


if __name__ == '__main__':
	mainloop = gobject.MainLoop()
	m = Manager('e4e4', '1183')
	try:
		mainloop.run()
	except KeyboardInterrupt:
		mainloop.quit()
		#print 'Exiting...'
		sys.exit(0)
